@@ -601,8 +601,8 @@ func (o *Operator) syncRegistryServer(logger *logrus.Entry, in *v1alpha1.Catalog
 	if healthy && in.Status.RegistryServiceStatus != nil {
 		logger.Debug("registry state good")
 		continueSync = true
-		// Check if registryService is ready for polling update
-		if !out.Update() {
+		// return here if catalog does not have polling enabled
+		if !out.Poll() {
 			return
 		}
 	}
@@ -610,10 +610,17 @@ func (o *Operator) syncRegistryServer(logger *logrus.Entry, in *v1alpha1.Catalog
 	// Registry pod hasn't been created or hasn't been updated since the last configmap update, recreate it
 	logger.Debug("ensuring registry server")
 
-	if err := srcReconciler.EnsureRegistryServer(out); err != nil {
-		syncError = fmt.Errorf("couldn't ensure registry server - %v", err)
-		out.SetError(v1alpha1.CatalogSourceRegistryServerError, syncError)
-		return
+	err = srcReconciler.EnsureRegistryServer(out)
+	if err != nil {
+		if _, ok := err.(reconciler.UpdateNotReadyErr); ok {
+			logger.Debug("requeueing registry server for catalog update check: update pod not yet ready")
+			o.catsrcQueueSet.RequeueAfter(out.GetNamespace(), out.GetName(), reconciler.CatalogPollingRequeuePeriod)
+			return
+		} else {
+			syncError = fmt.Errorf("couldn't ensure registry server - %v", err)
+			out.SetError(v1alpha1.CatalogSourceRegistryServerError, syncError)
+			return
+		}
 	}
 
 	logger.Debug("ensured registry server")
@@ -1556,7 +1563,7 @@ func (o *Operator) ExecutePlan(plan *v1alpha1.InstallPlan) error {
 
 	for i, step := range plan.Status.Plan {
 		doStep := true
-		s, err := b.create(step)
+		s, err := b.create(*step)
 		if err != nil {
 			if _, ok := err.(notSupportedStepperErr); ok {
 				// stepper not implemented for this type yet
