@@ -1,6 +1,7 @@
 package catalog
 
 import (
+	"context"
 	"fmt"
 
 	errorwrap "github.com/pkg/errors"
@@ -9,30 +10,34 @@ import (
 	apiequality "k8s.io/apimachinery/pkg/api/equality"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
+	"k8s.io/client-go/dynamic"
 
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/clientset/versioned"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorclient"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/ownerutil"
 )
 
-func newStepEnsurer(kubeClient operatorclient.ClientInterface, crClient versioned.Interface) *StepEnsurer {
+func newStepEnsurer(kubeClient operatorclient.ClientInterface, crClient versioned.Interface, dynamicClient dynamic.Interface) *StepEnsurer {
 	return &StepEnsurer{
-		kubeClient: kubeClient,
-		crClient:   crClient,
+		kubeClient:    kubeClient,
+		crClient:      crClient,
+		dynamicClient: dynamicClient,
 	}
 }
 
 // StepEnsurer ensures that resource(s) specified in install plan exist in cluster.
 type StepEnsurer struct {
-	kubeClient operatorclient.ClientInterface
-	crClient   versioned.Interface
+	kubeClient    operatorclient.ClientInterface
+	crClient      versioned.Interface
+	dynamicClient dynamic.Interface
 }
 
 // EnsureClusterServiceVersion writes the specified ClusterServiceVersion
 // object to the cluster.
 func (o *StepEnsurer) EnsureClusterServiceVersion(csv *v1alpha1.ClusterServiceVersion) (status v1alpha1.StepStatus, err error) {
-	_, createErr := o.crClient.OperatorsV1alpha1().ClusterServiceVersions(csv.GetNamespace()).Create(csv)
+	_, createErr := o.crClient.OperatorsV1alpha1().ClusterServiceVersions(csv.GetNamespace()).Create(context.TODO(), csv, metav1.CreateOptions{})
 	if createErr == nil {
 		status = v1alpha1.StepStatusCreated
 		return
@@ -49,7 +54,7 @@ func (o *StepEnsurer) EnsureClusterServiceVersion(csv *v1alpha1.ClusterServiceVe
 
 // EnsureSubscription writes the specified Subscription object to the cluster.
 func (o *StepEnsurer) EnsureSubscription(subscription *v1alpha1.Subscription) (status v1alpha1.StepStatus, err error) {
-	_, createErr := o.crClient.OperatorsV1alpha1().Subscriptions(subscription.GetNamespace()).Create(subscription)
+	_, createErr := o.crClient.OperatorsV1alpha1().Subscriptions(subscription.GetNamespace()).Create(context.TODO(), subscription, metav1.CreateOptions{})
 	if createErr == nil {
 		status = v1alpha1.StepStatusCreated
 		return
@@ -67,7 +72,7 @@ func (o *StepEnsurer) EnsureSubscription(subscription *v1alpha1.Subscription) (s
 // EnsureSecret copies the secret from the OLM namespace and writes a new one
 // to the namespace requested.
 func (o *StepEnsurer) EnsureSecret(operatorNamespace, planNamespace, name string) (status v1alpha1.StepStatus, err error) {
-	secret, getError := o.kubeClient.KubernetesInterface().CoreV1().Secrets(operatorNamespace).Get(name, metav1.GetOptions{})
+	secret, getError := o.kubeClient.KubernetesInterface().CoreV1().Secrets(operatorNamespace).Get(context.TODO(), name, metav1.GetOptions{})
 	if getError != nil {
 		if k8serrors.IsNotFound(getError) {
 			err = fmt.Errorf("secret %s does not exist - %v", name, getError)
@@ -91,7 +96,7 @@ func (o *StepEnsurer) EnsureSecret(operatorNamespace, planNamespace, name string
 		Type: secret.Type,
 	}
 
-	if _, createError := o.kubeClient.KubernetesInterface().CoreV1().Secrets(planNamespace).Create(newSecret); createError != nil {
+	if _, createError := o.kubeClient.KubernetesInterface().CoreV1().Secrets(planNamespace).Create(context.TODO(), newSecret, metav1.CreateOptions{}); createError != nil {
 		if k8serrors.IsAlreadyExists(createError) {
 			status = v1alpha1.StepStatusPresent
 			return
@@ -105,9 +110,33 @@ func (o *StepEnsurer) EnsureSecret(operatorNamespace, planNamespace, name string
 	return
 }
 
+// EnsureBundleSecret creates user-specified secrets from the bundle. Called when StepResource.Secret is true
+func (o *StepEnsurer) EnsureBundleSecret(namespace string, secret *corev1.Secret) (status v1alpha1.StepStatus, err error) {
+	_, createErr := o.kubeClient.KubernetesInterface().CoreV1().Secrets(namespace).Create(context.TODO(), secret, metav1.CreateOptions{})
+	if createErr == nil {
+		status = v1alpha1.StepStatusCreated
+		return
+	}
+
+	if !k8serrors.IsAlreadyExists(createErr) {
+		err = errorwrap.Wrapf(createErr, "error updating secret: %s", secret.GetName())
+		return
+	}
+
+	secret.SetNamespace(namespace)
+	// NOTE: any annotations/changes applied to the secret are lost
+	if _, updateErr := o.kubeClient.UpdateSecret(secret); updateErr != nil {
+		err = errorwrap.Wrapf(updateErr, "error updating secret: %s", secret.GetName())
+		return
+	}
+
+	status = v1alpha1.StepStatusPresent
+	return
+}
+
 // EnsureServiceAccount writes the specified ServiceAccount object to the cluster.
 func (o *StepEnsurer) EnsureServiceAccount(namespace string, sa *corev1.ServiceAccount) (status v1alpha1.StepStatus, err error) {
-	_, createErr := o.kubeClient.KubernetesInterface().CoreV1().ServiceAccounts(namespace).Create(sa)
+	_, createErr := o.kubeClient.KubernetesInterface().CoreV1().ServiceAccounts(namespace).Create(context.TODO(), sa, metav1.CreateOptions{})
 	if createErr == nil {
 		status = v1alpha1.StepStatusCreated
 		return
@@ -119,7 +148,8 @@ func (o *StepEnsurer) EnsureServiceAccount(namespace string, sa *corev1.ServiceA
 	}
 
 	// Carrying secrets through the service account update.
-	preSa, getErr := o.kubeClient.KubernetesInterface().CoreV1().ServiceAccounts(namespace).Get(sa.Name,
+	preSa, getErr := o.kubeClient.KubernetesInterface().CoreV1().ServiceAccounts(namespace).Get(context.TODO(),
+		sa.Name,
 		metav1.GetOptions{})
 	if getErr != nil {
 		err = errorwrap.Wrapf(getErr, "error getting older version of service account: %s", sa.GetName())
@@ -143,7 +173,7 @@ func (o *StepEnsurer) EnsureServiceAccount(namespace string, sa *corev1.ServiceA
 
 // EnsureService writes the specified Service object to the cluster.
 func (o *StepEnsurer) EnsureService(namespace string, service *corev1.Service) (status v1alpha1.StepStatus, err error) {
-	_, createErr := o.kubeClient.KubernetesInterface().CoreV1().Services(namespace).Create(service)
+	_, createErr := o.kubeClient.KubernetesInterface().CoreV1().Services(namespace).Create(context.TODO(), service, metav1.CreateOptions{})
 	if createErr == nil {
 		status = v1alpha1.StepStatusCreated
 		return
@@ -166,7 +196,7 @@ func (o *StepEnsurer) EnsureService(namespace string, service *corev1.Service) (
 
 // EnsureClusterRole writes the specified ClusterRole object to the cluster.
 func (o *StepEnsurer) EnsureClusterRole(cr *rbacv1.ClusterRole, step *v1alpha1.Step) (status v1alpha1.StepStatus, err error) {
-	_, createErr := o.kubeClient.KubernetesInterface().RbacV1().ClusterRoles().Create(cr)
+	_, createErr := o.kubeClient.KubernetesInterface().RbacV1().ClusterRoles().Create(context.TODO(), cr, metav1.CreateOptions{})
 	if createErr == nil {
 		status = v1alpha1.StepStatusCreated
 		return
@@ -193,7 +223,7 @@ func (o *StepEnsurer) EnsureClusterRole(cr *rbacv1.ClusterRole, step *v1alpha1.S
 
 // EnsureClusterRoleBinding writes the specified ClusterRoleBinding object to the cluster.
 func (o *StepEnsurer) EnsureClusterRoleBinding(crb *rbacv1.ClusterRoleBinding, step *v1alpha1.Step) (status v1alpha1.StepStatus, err error) {
-	_, createErr := o.kubeClient.KubernetesInterface().RbacV1().ClusterRoleBindings().Create(crb)
+	_, createErr := o.kubeClient.KubernetesInterface().RbacV1().ClusterRoleBindings().Create(context.TODO(), crb, metav1.CreateOptions{})
 	if createErr == nil {
 		status = v1alpha1.StepStatusCreated
 		return
@@ -220,7 +250,7 @@ func (o *StepEnsurer) EnsureClusterRoleBinding(crb *rbacv1.ClusterRoleBinding, s
 
 // EnsureRole writes the specified Role object to the cluster.
 func (o *StepEnsurer) EnsureRole(namespace string, role *rbacv1.Role) (status v1alpha1.StepStatus, err error) {
-	_, createErr := o.kubeClient.KubernetesInterface().RbacV1().Roles(namespace).Create(role)
+	_, createErr := o.kubeClient.KubernetesInterface().RbacV1().Roles(namespace).Create(context.TODO(), role, metav1.CreateOptions{})
 	if createErr == nil {
 		status = v1alpha1.StepStatusCreated
 		return
@@ -244,7 +274,7 @@ func (o *StepEnsurer) EnsureRole(namespace string, role *rbacv1.Role) (status v1
 
 // EnsureRoleBinding writes the specified RoleBinding object to the cluster.
 func (o *StepEnsurer) EnsureRoleBinding(namespace string, rb *rbacv1.RoleBinding) (status v1alpha1.StepStatus, err error) {
-	_, createErr := o.kubeClient.KubernetesInterface().RbacV1().RoleBindings(namespace).Create(rb)
+	_, createErr := o.kubeClient.KubernetesInterface().RbacV1().RoleBindings(namespace).Create(context.TODO(), rb, metav1.CreateOptions{})
 	if createErr == nil {
 		status = v1alpha1.StepStatusCreated
 		return
@@ -264,3 +294,58 @@ func (o *StepEnsurer) EnsureRoleBinding(namespace string, rb *rbacv1.RoleBinding
 	status = v1alpha1.StepStatusPresent
 	return
 }
+
+// EnsureUnstructuredObject writes the unspecified resource object to the cluster.
+func (o *StepEnsurer) EnsureUnstructuredObject(client dynamic.ResourceInterface, obj *unstructured.Unstructured) (status v1alpha1.StepStatus, err error) {
+	_, createErr := client.Create(context.TODO(), obj, metav1.CreateOptions{})
+	if createErr == nil {
+		status = v1alpha1.StepStatusCreated
+		return
+	}
+
+	if !k8serrors.IsAlreadyExists(createErr) {
+		err = errorwrap.Wrapf(createErr, "error creating unstructured object %s", obj.GetName())
+		return
+	}
+
+	original, getError := client.Get(context.TODO(), obj.GetName(), metav1.GetOptions{})
+	if getError != nil {
+		err = errorwrap.Wrapf(getError, "error getting unstructured object %s", obj.GetName())
+		return
+	}
+
+	// Set the objects resource version
+	obj.SetResourceVersion(original.GetResourceVersion())
+
+	_, updateError := client.Update(context.TODO(), obj, metav1.UpdateOptions{})
+	if updateError != nil {
+		err = errorwrap.Wrapf(updateError, "error updating unstructured object %s", obj.GetName())
+		return
+	}
+
+	status = v1alpha1.StepStatusPresent
+	return
+}
+
+// EnsureConfigMap writes the specified ConfigMap object to the cluster.
+func (o *StepEnsurer) EnsureConfigMap(namespace string, configmap *corev1.ConfigMap) (status v1alpha1.StepStatus, err error) {
+	_, createErr := o.kubeClient.KubernetesInterface().CoreV1().ConfigMaps(namespace).Create(context.TODO(), configmap, metav1.CreateOptions{})
+	if createErr == nil {
+		status = v1alpha1.StepStatusCreated
+		return
+	}
+
+	if !k8serrors.IsAlreadyExists(createErr) {
+		err = errorwrap.Wrapf(createErr, "error updating configmap: %s", configmap.GetName())
+		return
+	}
+
+	configmap.SetNamespace(namespace)
+	if _, updateErr := o.kubeClient.UpdateConfigMap(configmap); updateErr != nil {
+		err = errorwrap.Wrapf(updateErr, "error updating configmap: %s", configmap.GetName())
+		return
+	}
+
+	status = v1alpha1.StepStatusPresent
+	return
+}
