@@ -7,10 +7,11 @@ import (
 	"strings"
 	"time"
 
-	v1 "github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1"
+	v1 "github.com/operator-framework/api/pkg/operators/v1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/kubestate"
 	"github.com/sirupsen/logrus"
 	log "github.com/sirupsen/logrus"
+	admissionregistrationv1 "k8s.io/api/admissionregistration/v1"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
 	extinf "k8s.io/apiextensions-apiserver/pkg/client/informers/externalversions"
@@ -29,7 +30,7 @@ import (
 	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	kagg "k8s.io/kube-aggregator/pkg/client/informers/externalversions"
 
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/clientset/versioned"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/informers/externalversions"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/certs"
@@ -68,6 +69,7 @@ type Operator struct {
 	csvCopyQueueSet       *queueinformer.ResourceQueueSet
 	csvGCQueueSet         *queueinformer.ResourceQueueSet
 	objGCQueueSet         *queueinformer.ResourceQueueSet
+	nsQueueSet            workqueue.RateLimitingInterface
 	apiServiceQueue       workqueue.RateLimitingInterface
 	csvIndexers           map[string]cache.Indexer
 	recorder              record.EventRecorder
@@ -229,7 +231,9 @@ func newOperatorWithConfig(ctx context.Context, config *operatorConfig) (*Operat
 		if err != nil {
 			return nil, err
 		}
-		op.RegisterQueueInformer(subQueueInformer)
+		if err := op.RegisterQueueInformer(subQueueInformer); err != nil {
+			return nil, err
+		}
 
 		// Wire Deployments
 		k8sInformerFactory := informers.NewSharedInformerFactoryWithOptions(op.opClient.KubernetesInterface(), config.resyncPeriod(), informers.WithNamespace(namespace))
@@ -394,6 +398,7 @@ func newOperatorWithConfig(ctx context.Context, config *operatorConfig) (*Operat
 	// register namespace queueinformer
 	namespaceInformer := k8sInformerFactory.Core().V1().Namespaces()
 	op.lister.CoreV1().RegisterNamespaceLister(namespaceInformer.Lister())
+	op.nsQueueSet = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "resolver")
 	namespaceInformer.Informer().AddEventHandler(
 		&cache.ResourceEventHandlerFuncs{
 			DeleteFunc: op.namespaceAddedOrRemoved,
@@ -403,8 +408,9 @@ func newOperatorWithConfig(ctx context.Context, config *operatorConfig) (*Operat
 	namespaceQueueInformer, err := queueinformer.NewQueueInformer(
 		ctx,
 		queueinformer.WithLogger(op.logger),
+		queueinformer.WithQueue(op.nsQueueSet),
 		queueinformer.WithInformer(namespaceInformer.Informer()),
-		queueinformer.WithSyncer(queueinformer.LegacySyncHandler(op.syncObject).ToSyncer()),
+		queueinformer.WithSyncer(queueinformer.LegacySyncHandler(op.syncNamespace).ToSyncer()),
 	)
 	if err != nil {
 		return nil, err
@@ -431,8 +437,8 @@ func newOperatorWithConfig(ctx context.Context, config *operatorConfig) (*Operat
 	}
 
 	// Register CustomResourceDefinition QueueInformer
-	crdInformer := extinf.NewSharedInformerFactory(op.opClient.ApiextensionsV1beta1Interface(), config.resyncPeriod()).Apiextensions().V1beta1().CustomResourceDefinitions()
-	op.lister.APIExtensionsV1beta1().RegisterCustomResourceDefinitionLister(crdInformer.Lister())
+	crdInformer := extinf.NewSharedInformerFactory(op.opClient.ApiextensionsInterface(), config.resyncPeriod()).Apiextensions().V1().CustomResourceDefinitions()
+	op.lister.APIExtensionsV1().RegisterCustomResourceDefinitionLister(crdInformer.Lister())
 	crdQueueInformer, err := queueinformer.NewQueueInformer(
 		ctx,
 		queueinformer.WithLogger(op.logger),
@@ -476,7 +482,9 @@ func newOperatorWithConfig(ctx context.Context, config *operatorConfig) (*Operat
 		if err != nil {
 			return nil, err
 		}
-		op.RegisterQueueInformer(informer)
+		if err := op.RegisterQueueInformer(informer); err != nil {
+			return nil, err
+		}
 	}
 
 	overridesBuilderFunc := overrides.NewDeploymentInitializer(op.logger, proxyQuerierInUse, op.lister)
@@ -625,6 +633,44 @@ func (a *Operator) syncGCObject(obj interface{}) (syncError error) {
 			break
 		}
 		logger.Debugf("Deleted cluster role binding %v due to no owning CSV", metaObj.GetName())
+	case *admissionregistrationv1.MutatingWebhookConfiguration:
+		if name, ns, ok := ownerutil.GetOwnerByKindLabel(metaObj, v1alpha1.ClusterServiceVersionKind); ok {
+			_, err := a.lister.OperatorsV1alpha1().ClusterServiceVersionLister().ClusterServiceVersions(ns).Get(name)
+			if err == nil {
+				logger.Debugf("CSV still present, must wait until it is deleted (owners=%v)", name)
+				syncError = fmt.Errorf("cleanup must wait")
+				return
+			} else if !k8serrors.IsNotFound(err) {
+				logger.Infof("error CSV retrieval error")
+				syncError = err
+				return
+			}
+		}
+
+		if err := a.opClient.KubernetesInterface().AdmissionregistrationV1().MutatingWebhookConfigurations().Delete(context.TODO(), metaObj.GetName(), metav1.DeleteOptions{}); err != nil {
+			logger.WithError(err).Warn("cannot delete MutatingWebhookConfiguration")
+			break
+		}
+		logger.Debugf("Deleted MutatingWebhookConfiguration %v due to no owning CSV", metaObj.GetName())
+	case *admissionregistrationv1.ValidatingWebhookConfiguration:
+		if name, ns, ok := ownerutil.GetOwnerByKindLabel(metaObj, v1alpha1.ClusterServiceVersionKind); ok {
+			_, err := a.lister.OperatorsV1alpha1().ClusterServiceVersionLister().ClusterServiceVersions(ns).Get(name)
+			if err == nil {
+				logger.Debugf("CSV still present, must wait until it is deleted (owners=%v)", name)
+				syncError = fmt.Errorf("cleanup must wait")
+				return
+			} else if !k8serrors.IsNotFound(err) {
+				logger.Infof("Error CSV retrieval error")
+				syncError = err
+				return
+			}
+		}
+
+		if err := a.opClient.KubernetesInterface().AdmissionregistrationV1().ValidatingWebhookConfigurations().Delete(context.TODO(), metaObj.GetName(), metav1.DeleteOptions{}); err != nil {
+			logger.WithError(err).Warn("cannot delete ValidatingWebhookConfiguration")
+			break
+		}
+		logger.Debugf("Deleted ValidatingWebhookConfiguration %v due to no owning CSV", metaObj.GetName())
 	}
 
 	return
@@ -656,7 +702,7 @@ func (a *Operator) syncObject(obj interface{}) (syncError error) {
 			a.requeueOwnerCSVs(metaObj)
 		} else {
 			switch metaObj.(type) {
-			case *rbacv1.ClusterRole, *rbacv1.ClusterRoleBinding:
+			case *rbacv1.ClusterRole, *rbacv1.ClusterRoleBinding, *admissionregistrationv1.MutatingWebhookConfiguration, *admissionregistrationv1.ValidatingWebhookConfiguration:
 				resourceEvent := kubestate.NewResourceEvent(
 					kubestate.ResourceUpdated,
 					metaObj,
@@ -694,7 +740,7 @@ func (a *Operator) syncObject(obj interface{}) (syncError error) {
 				continue
 			}
 			csv.SetPhase(v1alpha1.CSVPhasePending, v1alpha1.CSVReasonDetectedClusterChange, "Cluster resources changed state", a.now())
-			_, err := a.client.OperatorsV1alpha1().ClusterServiceVersions(csv.GetNamespace()).UpdateStatus(csv)
+			_, err := a.client.OperatorsV1alpha1().ClusterServiceVersions(csv.GetNamespace()).UpdateStatus(context.TODO(), csv, metav1.UpdateOptions{})
 			if err != nil {
 				errs = append(errs, err)
 				continue
@@ -737,6 +783,53 @@ func (a *Operator) namespaceAddedOrRemoved(obj interface{}) {
 	return
 }
 
+func (a *Operator) syncNamespace(obj interface{}) error {
+	// Check to see if any operator groups are associated with this namespace
+	namespace, ok := obj.(*corev1.Namespace)
+	if !ok {
+		a.logger.Debugf("wrong type: %#v\n", obj)
+		return fmt.Errorf("casting Namespace failed")
+	}
+
+	logger := a.logger.WithFields(logrus.Fields{
+		"name": namespace.GetName(),
+	})
+
+	// Remove existing OperatorGroup labels
+	for label := range namespace.GetLabels() {
+		if v1.IsOperatorGroupLabel(label) {
+			delete(namespace.Labels, label)
+		}
+	}
+
+	operatorGroupList, err := a.lister.OperatorsV1().OperatorGroupLister().List(labels.Everything())
+	if err != nil {
+		logger.WithError(err).Warn("lister failed")
+		return err
+	}
+
+	for _, group := range operatorGroupList {
+		namespaceSet := resolver.NewNamespaceSet(group.Status.Namespaces)
+
+		// Apply the label if not an All Namespaces OperatorGroup.
+		if namespaceSet.Contains(namespace.GetName()) && !namespaceSet.IsAllNamespaces() {
+			if namespace.Labels == nil {
+				namespace.Labels = make(map[string]string, 1)
+			}
+			ogLabelKey, ogLabelValue, err := group.OGLabelKeyAndValue()
+			if err != nil {
+				return err
+			}
+			namespace.Labels[ogLabelKey] = ogLabelValue
+		}
+	}
+
+	// Update the Namespace
+	_, err = a.opClient.KubernetesInterface().CoreV1().Namespaces().Update(context.TODO(), namespace, metav1.UpdateOptions{})
+
+	return err
+}
+
 func (a *Operator) handleClusterServiceVersionDeletion(obj interface{}) {
 	clusterServiceVersion, ok := obj.(*v1alpha1.ClusterServiceVersion)
 	if !ok {
@@ -812,9 +905,9 @@ func (a *Operator) handleClusterServiceVersionDeletion(obj interface{}) {
 	}
 
 	logger.Info("gcing children")
-	namespaces := []string{}
+	namespaces := make([]string, 0)
 	if targetNamespaces == "" {
-		namespaceList, err := a.opClient.KubernetesInterface().CoreV1().Namespaces().List(metav1.ListOptions{})
+		namespaceList, err := a.opClient.KubernetesInterface().CoreV1().Namespaces().List(context.TODO(), metav1.ListOptions{})
 		if err != nil {
 			logger.WithError(err).Warn("cannot list all namespaces to requeue child csvs for deletion")
 			return
@@ -828,7 +921,9 @@ func (a *Operator) handleClusterServiceVersionDeletion(obj interface{}) {
 	for _, namespace := range namespaces {
 		if namespace != operatorNamespace {
 			logger.WithField("targetNamespace", namespace).Debug("requeueing child csv for deletion")
-			a.csvGCQueueSet.Requeue(namespace, clusterServiceVersion.GetName())
+			if err := a.csvGCQueueSet.Requeue(namespace, clusterServiceVersion.GetName()); err != nil {
+				logger.WithError(err).Warn("unable to requeue")
+			}
 		}
 	}
 
@@ -872,6 +967,29 @@ func (a *Operator) handleClusterServiceVersionDeletion(obj interface{}) {
 			logger.WithError(err).Warnf("failed to requeue gc event: %v", cr)
 		}
 	}
+
+	webhookSelector := labels.SelectorFromSet(ownerutil.OwnerLabel(clusterServiceVersion, v1alpha1.ClusterServiceVersionKind)).String()
+	mWebhooks, err := a.opClient.KubernetesInterface().AdmissionregistrationV1().MutatingWebhookConfigurations().List(context.TODO(), metav1.ListOptions{LabelSelector: webhookSelector})
+	if err != nil {
+		logger.WithError(err).Warn("cannot list MutatingWebhookConfigurations")
+	}
+	for _, webhook := range mWebhooks.Items {
+		w := webhook
+		if err := a.objGCQueueSet.RequeueEvent("", kubestate.NewResourceEvent(kubestate.ResourceUpdated, &w)); err != nil {
+			logger.WithError(err).Warnf("failed to requeue gc event: %v", webhook)
+		}
+	}
+
+	vWebhooks, err := a.opClient.KubernetesInterface().AdmissionregistrationV1().ValidatingWebhookConfigurations().List(context.TODO(), metav1.ListOptions{LabelSelector: webhookSelector})
+	if err != nil {
+		logger.WithError(err).Warn("cannot list ValidatingWebhookConfigurations")
+	}
+	for _, webhook := range vWebhooks.Items {
+		w := webhook
+		if err := a.objGCQueueSet.RequeueEvent("", kubestate.NewResourceEvent(kubestate.ResourceUpdated, &w)); err != nil {
+			logger.WithError(err).Warnf("failed to requeue gc event: %v", webhook)
+		}
+	}
 }
 
 func (a *Operator) removeDanglingChildCSVs(csv *v1alpha1.ClusterServiceVersion) error {
@@ -925,7 +1043,7 @@ func (a *Operator) removeDanglingChildCSVs(csv *v1alpha1.ClusterServiceVersion)
 
 func (a *Operator) deleteChild(csv *v1alpha1.ClusterServiceVersion, logger *logrus.Entry) error {
 	logger.Debug("gcing csv")
-	return a.client.OperatorsV1alpha1().ClusterServiceVersions(csv.GetNamespace()).Delete(csv.GetName(), metav1.NewDeleteOptions(0))
+	return a.client.OperatorsV1alpha1().ClusterServiceVersions(csv.GetNamespace()).Delete(context.TODO(), csv.GetName(), *metav1.NewDeleteOptions(0))
 }
 
 // syncClusterServiceVersion is the method that gets called when we see a CSV event in the cluster
@@ -950,7 +1068,9 @@ func (a *Operator) syncClusterServiceVersion(obj interface{}) (syncError error)
 
 	if clusterServiceVersion.IsCopied() {
 		logger.Debug("skipping copied csv transition, schedule for gc check")
-		a.csvGCQueueSet.Requeue(clusterServiceVersion.GetNamespace(), clusterServiceVersion.GetName())
+		if err := a.csvGCQueueSet.Requeue(clusterServiceVersion.GetNamespace(), clusterServiceVersion.GetName()); err != nil {
+			logger.WithError(err).Warn("unable to requeue")
+		}
 		return
 	}
 
@@ -967,7 +1087,7 @@ func (a *Operator) syncClusterServiceVersion(obj interface{}) (syncError error)
 		outCSV.Status.Message == clusterServiceVersion.Status.Message) {
 
 		// Update CSV with status of transition. Log errors if we can't write them to the status.
-		_, err := a.client.OperatorsV1alpha1().ClusterServiceVersions(outCSV.GetNamespace()).UpdateStatus(outCSV)
+		_, err := a.client.OperatorsV1alpha1().ClusterServiceVersions(outCSV.GetNamespace()).UpdateStatus(context.TODO(), outCSV, metav1.UpdateOptions{})
 		if err != nil {
 			updateErr := errors.New("error updating ClusterServiceVersion status: " + err.Error())
 			if syncError == nil {
@@ -999,7 +1119,9 @@ func (a *Operator) syncClusterServiceVersion(obj interface{}) (syncError error)
 	}
 
 	if !outCSV.IsUncopiable() {
-		a.csvCopyQueueSet.Requeue(outCSV.GetNamespace(), outCSV.GetName())
+		if err := a.csvCopyQueueSet.Requeue(outCSV.GetNamespace(), outCSV.GetName()); err != nil {
+			logger.WithError(err).Warn("unable to requeue")
+		}
 	}
 
 	logger.Debug("done syncing CSV")
@@ -1128,7 +1250,7 @@ func (a *Operator) operatorGroupForCSV(csv *v1alpha1.ClusterServiceVersion, logg
 		logger = logger.WithField("opgroup", operatorGroup.GetName())
 		if a.operatorGroupAnnotationsDiffer(&csv.ObjectMeta, operatorGroup) {
 			a.setOperatorGroupAnnotations(&csv.ObjectMeta, operatorGroup, true)
-			if _, err := a.client.OperatorsV1alpha1().ClusterServiceVersions(csv.GetNamespace()).Update(csv); err != nil {
+			if _, err := a.client.OperatorsV1alpha1().ClusterServiceVersions(csv.GetNamespace()).Update(context.TODO(), csv, metav1.UpdateOptions{}); err != nil {
 				logger.WithError(err).Warn("error adding operatorgroup annotations")
 				return nil, err
 			}
@@ -1273,10 +1395,12 @@ func (a *Operator) transitionCSVState(in v1alpha1.ClusterServiceVersion) (out *v
 		}
 		unionedAnnotations[v1.OperatorGroupProvidedAPIsAnnotationKey] = union.String()
 		operatorGroup.SetAnnotations(unionedAnnotations)
-		if _, err := a.client.OperatorsV1().OperatorGroups(operatorGroup.GetNamespace()).Update(operatorGroup); err != nil && !k8serrors.IsNotFound(err) {
+		if _, err := a.client.OperatorsV1().OperatorGroups(operatorGroup.GetNamespace()).Update(context.TODO(), operatorGroup, metav1.UpdateOptions{}); err != nil && !k8serrors.IsNotFound(err) {
 			syncError = fmt.Errorf("could not update operatorgroups %s annotation: %v", v1.OperatorGroupProvidedAPIsAnnotationKey, err)
 		}
-		a.csvQueueSet.Requeue(out.GetNamespace(), out.GetName())
+		if err := a.csvQueueSet.Requeue(out.GetNamespace(), out.GetName()); err != nil {
+			a.logger.WithError(err).Warn("unable to requeue")
+		}
 		return
 	case result == resolver.RemoveAPIs:
 		// Remove the CSV's provided APIs from its OperatorGroup's annotation
@@ -1285,11 +1409,13 @@ func (a *Operator) transitionCSVState(in v1alpha1.ClusterServiceVersion) (out *v
 		if diffedAnnotations := operatorGroup.GetAnnotations(); diffedAnnotations != nil {
 			diffedAnnotations[v1.OperatorGroupProvidedAPIsAnnotationKey] = difference.String()
 			operatorGroup.SetAnnotations(diffedAnnotations)
-			if _, err := a.client.OperatorsV1().OperatorGroups(operatorGroup.GetNamespace()).Update(operatorGroup); err != nil && !k8serrors.IsNotFound(err) {
+			if _, err := a.client.OperatorsV1().OperatorGroups(operatorGroup.GetNamespace()).Update(context.TODO(), operatorGroup, metav1.UpdateOptions{}); err != nil && !k8serrors.IsNotFound(err) {
 				syncError = fmt.Errorf("could not update operatorgroups %s annotation: %v", v1.OperatorGroupProvidedAPIsAnnotationKey, err)
 			}
 		}
-		a.csvQueueSet.Requeue(out.GetNamespace(), out.GetName())
+		if err := a.csvQueueSet.Requeue(out.GetNamespace(), out.GetName()); err != nil {
+			a.logger.WithError(err).Warn("unable to requeue")
+		}
 		return
 	default:
 		logger.WithField("apis", providedAPIs).Debug("no intersecting operatorgroups provide the same apis")
@@ -1325,6 +1451,22 @@ func (a *Operator) transitionCSVState(in v1alpha1.ClusterServiceVersion) (out *v
 			return
 		}
 
+		// Create a map to track unique names
+		webhookNames := map[string]struct{}{}
+		// Check if Webhooks have valid rules and unique names
+		for _, desc := range out.Spec.WebhookDefinitions {
+			_, present := webhookNames[desc.GenerateName]
+			if present {
+				out.SetPhaseWithEventIfChanged(v1alpha1.CSVPhaseFailed, v1alpha1.CSVReasonInvalidWebhookDescription, "CSV contains repeated WebhookDescription name", now, a.recorder)
+				return
+			}
+			webhookNames[desc.GenerateName] = struct{}{}
+			if syncError = install.ValidWebhookRules(desc.Rules); syncError != nil {
+				out.SetPhaseWithEventIfChanged(v1alpha1.CSVPhaseFailed, v1alpha1.CSVReasonInvalidWebhookDescription, syncError.Error(), now, a.recorder)
+				return
+			}
+		}
+
 		// Check for CRD ownership conflicts
 		if syncError = a.crdOwnerConflicts(out, a.csvSet(out.GetNamespace(), v1alpha1.CSVPhaseAny)); syncError != nil {
 			if syncError == ErrCRDOwnerConflict {
@@ -1356,13 +1498,6 @@ func (a *Operator) transitionCSVState(in v1alpha1.ClusterServiceVersion) (out *v
 			return
 		}
 
-		// Install owned APIServices and update strategy with serving cert data
-		strategy, syncError = a.installOwnedAPIServiceRequirements(out, strategy)
-		if syncError != nil {
-			out.SetPhaseWithEvent(v1alpha1.CSVPhaseFailed, v1alpha1.CSVReasonComponentFailed, fmt.Sprintf("install API services failed: %s", syncError), now, a.recorder)
-			return
-		}
-
 		if syncError = installer.Install(strategy); syncError != nil {
 			if install.IsErrorUnrecoverable(syncError) {
 				logger.Infof("Setting CSV reason to failed without retry: %v", syncError)
@@ -1373,6 +1508,15 @@ func (a *Operator) transitionCSVState(in v1alpha1.ClusterServiceVersion) (out *v
 			return
 		}
 
+		if out.HasCAResources() {
+			now := metav1.Now()
+			expiration := now.Add(install.DefaultCertValidFor)
+			rotateAt := expiration.Add(-1 * install.DefaultCertMinFresh)
+			rotateTime := metav1.NewTime(rotateAt)
+			out.Status.CertsLastUpdated = &now
+			out.Status.CertsRotateAt = &rotateTime
+		}
+
 		out.SetPhaseWithEvent(v1alpha1.CSVPhaseInstalling, v1alpha1.CSVReasonInstallSuccessful, "waiting for install components to report healthy", now, a.recorder)
 		err := a.csvQueueSet.Requeue(out.GetNamespace(), out.GetName())
 		if err != nil {
@@ -1449,7 +1593,7 @@ func (a *Operator) transitionCSVState(in v1alpha1.ClusterServiceVersion) (out *v
 		}
 
 		// Ensure cluster roles exist for using provided apis
-		if err := a.ensureClusterRolesForCSV(out, operatorGroup); err != nil {
+		if err := a.ensureClusterRolesForCSV(out); err != nil {
 			logger.WithError(err).Info("couldn't ensure clusterroles for provided api types")
 			syncError = err
 			return
@@ -1550,10 +1694,10 @@ func (a *Operator) transitionCSVState(in v1alpha1.ClusterServiceVersion) (out *v
 				}
 			}
 		} else {
-			syncError = fmt.Errorf("CSV marked as replacement, but no replacement CSV found in cluster.")
+			syncError = fmt.Errorf("marked as replacement, but no replacement CSV found in cluster")
 		}
 	case v1alpha1.CSVPhaseDeleting:
-		syncError = a.client.OperatorsV1alpha1().ClusterServiceVersions(out.GetNamespace()).Delete(out.GetName(), metav1.NewDeleteOptions(0))
+		syncError = a.client.OperatorsV1alpha1().ClusterServiceVersions(out.GetNamespace()).Delete(context.TODO(), out.GetName(), *metav1.NewDeleteOptions(0))
 		if syncError != nil {
 			logger.Debugf("unable to get delete csv marked for deletion: %s", syncError.Error())
 		}
@@ -1573,7 +1717,7 @@ func (a *Operator) checkReplacementsAndUpdateStatus(csv *v1alpha1.ClusterService
 		return nil
 	}
 	if replacement := a.isBeingReplaced(csv, a.csvSet(csv.GetNamespace(), v1alpha1.CSVPhaseAny)); replacement != nil {
-		a.logger.Infof("newer csv replacing %s, no-op", csv.SelfLink)
+		a.logger.Infof("newer csv replacing %s, no-op", csv.GetName())
 		msg := fmt.Sprintf("being replaced by csv: %s", replacement.GetName())
 		csv.SetPhaseWithEvent(v1alpha1.CSVPhaseReplacing, v1alpha1.CSVReasonBeingReplaced, msg, a.now(), a.recorder)
 		metrics.CSVUpgradeCount.Inc()
@@ -1584,7 +1728,6 @@ func (a *Operator) checkReplacementsAndUpdateStatus(csv *v1alpha1.ClusterService
 }
 
 func (a *Operator) updateInstallStatus(csv *v1alpha1.ClusterServiceVersion, installer install.StrategyInstaller, strategy install.Strategy, requeuePhase v1alpha1.ClusterServiceVersionPhase, requeueConditionReason v1alpha1.ConditionReason) error {
-	apiServicesInstalled, apiServiceErr := a.areAPIServicesAvailable(csv)
 	strategyInstalled, strategyErr := installer.CheckInstalled(strategy)
 	now := a.now()
 
@@ -1592,7 +1735,10 @@ func (a *Operator) updateInstallStatus(csv *v1alpha1.ClusterServiceVersion, inst
 		a.logger.WithError(strategyErr).Debug("operator not installed")
 	}
 
-	if strategyInstalled && apiServicesInstalled {
+	apiServicesInstalled, apiServiceErr := a.areAPIServicesAvailable(csv)
+	webhooksInstalled, webhookErr := a.areWebhooksAvailable(csv)
+
+	if strategyInstalled && apiServicesInstalled && webhooksInstalled {
 		// if there's no error, we're successfully running
 		csv.SetPhaseWithEventIfChanged(v1alpha1.CSVPhaseSucceeded, v1alpha1.CSVReasonInstallSuccessful, "install strategy completed with no errors", now, a.recorder)
 		return nil
@@ -1618,6 +1764,16 @@ func (a *Operator) updateInstallStatus(csv *v1alpha1.ClusterServiceVersion, inst
 		return fmt.Errorf("APIServices not installed")
 	}
 
+	if !webhooksInstalled || webhookErr != nil {
+		msg := "Webhooks not installed"
+		csv.SetPhaseWithEventIfChanged(requeuePhase, requeueConditionReason, fmt.Sprintf(msg), now, a.recorder)
+		if err := a.csvQueueSet.Requeue(csv.GetNamespace(), csv.GetName()); err != nil {
+			a.logger.Warn(err.Error())
+		}
+
+		return fmt.Errorf(msg)
+	}
+
 	if strategyErr != nil {
 		if install.ReasonForError(strategyErr) == install.StrategyErrDeploymentUpdated {
 			csv.SetPhaseWithEventIfChanged(v1alpha1.CSVPhaseInstallReady, requeueConditionReason, fmt.Sprintf("installing: %s", strategyErr), now, a.recorder)
@@ -1667,7 +1823,7 @@ func (a *Operator) parseStrategiesAndUpdateStatus(csv *v1alpha1.ClusterServiceVe
 	}
 
 	strName := strategy.GetStrategyName()
-	installer := a.resolver.InstallerForStrategy(strName, kubeclient, a.lister, csv, csv.GetAnnotations(), previousStrategy)
+	installer := a.resolver.InstallerForStrategy(strName, kubeclient, a.lister, csv, csv.GetAnnotations(), csv.GetAllAPIServiceDescriptions(), csv.Spec.WebhookDefinitions, previousStrategy)
 	return installer, strategy
 }
 
@@ -1747,7 +1903,7 @@ func (a *Operator) apiServiceOwnerConflicts(csv *v1alpha1.ClusterServiceVersion)
 			continue
 		}
 
-		adoptable, err := a.isAPIServiceAdoptable(csv, apiService)
+		adoptable, err := install.IsAPIServiceAdoptable(a.lister, csv, apiService)
 		if err != nil {
 			a.logger.WithFields(log.Fields{"obj": "apiService", "labels": apiService.GetLabels()}).Errorf("adoption check failed - %v", err)
 		}
@@ -1773,13 +1929,13 @@ func (a *Operator) handleDeletion(obj interface{}) {
 	if !ok {
 		tombstone, ok := obj.(cache.DeletedFinalStateUnknown)
 		if !ok {
-			utilruntime.HandleError(fmt.Errorf("Couldn't get object from tombstone %#v", obj))
+			utilruntime.HandleError(fmt.Errorf("couldn't get object from tombstone %#v", obj))
 			return
 		}
 
 		metaObj, ok = tombstone.Obj.(metav1.Object)
 		if !ok {
-			utilruntime.HandleError(fmt.Errorf("Tombstone contained object that is not a metav1.Object %#v", obj))
+			utilruntime.HandleError(fmt.Errorf("tombstone contained object that is not a metav1.Object %#v", obj))
 			return
 		}
 	}
@@ -1899,6 +2055,6 @@ func (a *Operator) ensureLabels(in *v1alpha1.ClusterServiceVersion, labelSets ..
 
 	out := in.DeepCopy()
 	out.SetLabels(merged)
-	out, err := a.client.OperatorsV1alpha1().ClusterServiceVersions(out.GetNamespace()).Update(out)
+	out, err := a.client.OperatorsV1alpha1().ClusterServiceVersions(out.GetNamespace()).Update(context.TODO(), out, metav1.UpdateOptions{})
 	return out, err
 }
