@@ -3,7 +3,7 @@ package operators
 import (
 	"encoding/json"
 
-	operatorsv1alpha1 "github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	operatorsv1alpha1 "github.com/operator-framework/api/pkg/operators/v1alpha1"
 	"github.com/operator-framework/operator-registry/pkg/registry"
 )
 
@@ -32,11 +32,11 @@ func CreateCSVDescription(csv *operatorsv1alpha1.ClusterServiceVersion, csvJSON
 			Owned:    descriptionsForAPIServices(csv.Spec.APIServiceDefinitions.Owned),
 			Required: descriptionsForAPIServices(csv.Spec.APIServiceDefinitions.Required),
 		},
-		Keywords:       csv.Spec.Keywords,
-		Maturity:       csv.Spec.Maturity,
 		NativeAPIs:     csv.Spec.NativeAPIs,
 		MinKubeVersion: csv.Spec.MinKubeVersion,
 		RelatedImages:  GetImages(csvJSON),
+		Keywords:       csv.Spec.Keywords,
+		Maturity:       csv.Spec.Maturity,
 	}
 
 	icons := make([]Icon, len(csv.Spec.Icon))
